@@ -1,28 +1,30 @@
 """
 UBXReader class.
 
-Reads and parses individual UBX, NMEA or RTCM3 messages from any stream
-which supports a read(n) -> bytes method.
+Reads and parses individual UBX, NMEA or RTCM3 messages from any viable
+data stream which supports a read(n) -> bytes method.
 
 Returns both the raw binary data (as bytes) and the parsed data
 (as a UBXMessage, NMEAMessage or RTCMMessage object).
 
-'protfilter' governs which protocols (NMEA, UBX or RTCM3) are processed
-'quitonerror' governs how errors are handled
-'msgmode' indicates the type of UBX datastream (input GET, output SET, query POLL)
+- 'protfilter' governs which protocols (NMEA, UBX or RTCM3) are processed
+- 'quitonerror' governs how errors are handled
+- 'msgmode' indicates the type of UBX datastream (output GET, input SET, query POLL).
+  If msgmode is set to SETPOLL, input/query mode will be automatically detected by parser.
 
 Created on 2 Oct 2020
 
-:author: semuadmin
-:copyright: SEMU Consulting © 2020
+:author: semuadmin (Steve Smith)
+:copyright: semuadmin © 2020
 :license: BSD 3-Clause
 """
 
+from logging import getLogger
 from socket import socket
 
 import pynmeagps.exceptions as nme
 import pyrtcm.exceptions as rte
-from pynmeagps import NMEA_HDR, NMEAReader
+from pynmeagps import NMEA_HDR, NMEAReader, SocketWrapper
 from pyrtcm import RTCMReader
 
 from pyubx2.exceptions import (
@@ -31,15 +33,17 @@ from pyubx2.exceptions import (
     UBXStreamError,
     UBXTypeError,
 )
-from pyubx2.socket_stream import SocketStream
-from pyubx2.ubxhelpers import bytes2val, calc_checksum, val2bytes
+from pyubx2.ubxhelpers import bytes2val, calc_checksum, getinputmode, val2bytes
 from pyubx2.ubxmessage import UBXMessage
 from pyubx2.ubxtypes_core import (
     ERR_LOG,
     ERR_RAISE,
     GET,
     NMEA_PROTOCOL,
+    POLL,
     RTCM3_PROTOCOL,
+    SET,
+    SETPOLL,
     U2,
     UBX_HDR,
     UBX_PROTOCOL,
@@ -57,11 +61,10 @@ class UBXReader:
         datastream,
         msgmode: int = GET,
         validate: int = VALCKSUM,
-        protfilter: int = NMEA_PROTOCOL | UBX_PROTOCOL,
+        protfilter: int = NMEA_PROTOCOL | UBX_PROTOCOL | RTCM3_PROTOCOL,
         quitonerror: int = ERR_LOG,
         parsebitfield: bool = True,
-        scaling: bool = True,
-        labelmsm: bool = True,
+        labelmsm: int = 1,
         bufsize: int = 4096,
         parsing: bool = True,
         errorhandler: object = None,
@@ -69,22 +72,24 @@ class UBXReader:
         """Constructor.
 
         :param datastream stream: input data stream
-        :param int msgmode: 0=GET, 1=SET, 2=POLL (0)
-        :param int validate: 0 = ignore invalid checksum, 1 = validate checksum (1)
-        :param int protfilter: protocol filter 1 = NMEA, 2 = UBX, 4 = RTCM3 (3)
-        :param int quitonerror: 0 = ignore errors,  1 = log continue, 2 = (re)raise (1)
+        :param int msgmode: 0=GET, 1=SET, 2=POLL, 3=SETPOLL (0)
+        :param int validate: VALCKSUM (1) = Validate checksum,
+            VALNONE (0) = ignore invalid checksum (1)
+        :param int protfilter: NMEA_PROTOCOL (1), UBX_PROTOCOL (2), RTCM3_PROTOCOL (4),
+            Can be OR'd (7)
+        :param int quitonerror: ERR_IGNORE (0) = ignore errors,  ERR_LOG (1) = log continue,
+            ERR_RAISE (2) = (re)raise (1)
         :param bool parsebitfield: 1 = parse bitfields, 0 = leave as bytes (1)
-        :param bool scaling: 1 = apply scale factors, 0 = do not apply (1)
-        :param bool labelmsm: whether to label RTCM3 MSM NSAT and NCELL attributes (1)
-        :param int bufsize: socket recv buffer size (1024)
+        :param int labelmsm: RTCM3 MSM label type 1 = RINEX, 2 = BAND (1)
+        :param int bufsize: socket recv buffer size (4096)
         :param bool parsing: True = parse data, False = don't parse data (output raw only) (True)
-        :param int errorhandler: error handling object or function (None)
+        :param object errorhandler: error handling object or function (None)
         :raises: UBXStreamError (if mode is invalid)
         """
         # pylint: disable=too-many-arguments
 
         if isinstance(datastream, socket):
-            self._stream = SocketStream(datastream, bufsize=bufsize)
+            self._stream = SocketWrapper(datastream, bufsize=bufsize)
         else:
             self._stream = datastream
         self._protfilter = protfilter
@@ -92,14 +97,14 @@ class UBXReader:
         self._errorhandler = errorhandler
         self._validate = validate
         self._parsebf = parsebitfield
-        self._scaling = scaling
         self._labelmsm = labelmsm
         self._msgmode = msgmode
         self._parsing = parsing
+        self._logger = getLogger(__name__)
 
-        if self._msgmode not in (0, 1, 2):
+        if self._msgmode not in (GET, SET, POLL, SETPOLL):
             raise UBXStreamError(
-                f"Invalid stream mode {self._msgmode} - must be 0, 1 or 2"
+                f"Invalid stream mode {self._msgmode} - must be 0, 1, 2 or 3"
             )
 
     def __iter__(self):
@@ -132,13 +137,13 @@ class UBXReader:
 
         :return: tuple of (raw_data as bytes, parsed_data as UBXMessage, NMEAMessage or RTCMMessage)
         :rtype: tuple
-        :raises: UBXStreamError (if unrecognised protocol in data stream)
+        :raises: Exception (if invalid or unrecognised protocol in data stream)
         """
 
-        flag = True
+        parsing = True
+        while parsing:  # loop until end of valid message or EOF
+            try:
 
-        try:
-            while flag:  # loop until end of valid message or EOF
                 raw_data = None
                 parsed_data = None
                 byte1 = self._read_bytes(1)  # read the first byte
@@ -153,16 +158,16 @@ class UBXReader:
                     # if protocol filter passes UBX, return message,
                     # otherwise discard and continue
                     if self._protfilter & UBX_PROTOCOL:
-                        flag = False
+                        parsing = False
                     else:
                         continue
-                # if it's an NMEA message ('$G' or '$P')
+                # if it's an NMEA message (b'\x24\x..)
                 elif bytehdr in NMEA_HDR:
                     (raw_data, parsed_data) = self._parse_nmea(bytehdr)
                     # if protocol filter passes NMEA, return message,
                     # otherwise discard and continue
                     if self._protfilter & NMEA_PROTOCOL:
-                        flag = False
+                        parsing = False
                     else:
                         continue
                 # if it's a RTCM3 message
@@ -172,36 +177,32 @@ class UBXReader:
                     # if protocol filter passes RTCM, return message,
                     # otherwise discard and continue
                     if self._protfilter & RTCM3_PROTOCOL:
-                        flag = False
+                        parsing = False
                     else:
                         continue
                 # unrecognised protocol header
                 else:
-                    if self._quitonerror == ERR_RAISE:
-                        raise UBXParseError(f"Unknown protocol {bytehdr}.")
-                    if self._quitonerror == ERR_LOG:
-                        return (bytehdr, f"<UNKNOWN PROTOCOL(header={bytehdr})>")
-                    continue
+                    raise UBXParseError(f"Unknown protocol header {bytehdr}.")
 
-        except EOFError:
-            return (None, None)
-        except (
-            UBXMessageError,
-            UBXTypeError,
-            UBXParseError,
-            UBXStreamError,
-            nme.NMEAMessageError,
-            nme.NMEATypeError,
-            nme.NMEAParseError,
-            nme.NMEAStreamError,
-            rte.RTCMMessageError,
-            rte.RTCMParseError,
-            rte.RTCMStreamError,
-            rte.RTCMTypeError,
-        ) as err:
-            if self._quitonerror:
-                self._do_error(str(err))
-            parsed_data = str(err)
+            except EOFError:
+                return (None, None)
+            except (
+                UBXMessageError,
+                UBXTypeError,
+                UBXParseError,
+                UBXStreamError,
+                nme.NMEAMessageError,
+                nme.NMEATypeError,
+                nme.NMEAParseError,
+                nme.NMEAStreamError,
+                rte.RTCMMessageError,
+                rte.RTCMParseError,
+                rte.RTCMStreamError,
+                rte.RTCMTypeError,
+            ) as err:
+                if self._quitonerror:
+                    self._do_error(err)
+                continue
 
         return (raw_data, parsed_data)
 
@@ -209,7 +210,7 @@ class UBXReader:
         """
         Parse remainder of UBX message.
 
-        :param bytes hdr: UBX header (b'\xb5\x62')
+        :param bytes hdr: UBX header (b'\\xb5\\x62')
         :return: tuple of (raw_data as bytes, parsed_data as UBXMessage or None)
         :rtype: tuple
         """
@@ -229,10 +230,8 @@ class UBXReader:
             parsed_data = self.parse(
                 raw_data,
                 validate=self._validate,
-                quitonerror=self._quitonerror,
                 msgmode=self._msgmode,
                 parsebitfield=self._parsebf,
-                scaling=self._scaling,
             )
         else:
             parsed_data = None
@@ -242,7 +241,7 @@ class UBXReader:
         """
         Parse remainder of NMEA message (using pynmeagps library).
 
-        :param bytes hdr: NMEA header ($G or $P)
+        :param bytes hdr: NMEA header (b'\\x24\\x..')
         :return: tuple of (raw_data as bytes, parsed_data as NMEAMessage or None)
         :rtype: tuple
         """
@@ -267,7 +266,6 @@ class UBXReader:
         Parse any RTCM3 data in the stream (using pyrtcm library).
 
         :param bytes hdr: first 2 bytes of RTCM3 header
-        :param bool validate: (kwarg) validate crc Y/N
         :return: tuple of (raw_data as bytes, parsed_stub as RTCMMessage)
         :rtype: tuple
         """
@@ -283,7 +281,6 @@ class UBXReader:
             parsed_data = RTCMReader.parse(
                 raw_data,
                 validate=self._validate,
-                scaling=self._scaling,
                 labelmsm=self._labelmsm,
             )
         else:
@@ -297,12 +294,17 @@ class UBXReader:
         :param int size: number of bytes to read
         :return: bytes
         :rtype: bytes
-        :raises: EOFError if stream ends prematurely
+        :raises: UBXStreamError if stream ends prematurely
         """
 
         data = self._stream.read(size)
-        if len(data) < size:  # EOF
+        if len(data) == 0:  # EOF
             raise EOFError()
+        if 0 < len(data) < size:  # truncated stream
+            raise UBXStreamError(
+                "Serial stream terminated unexpectedly. "
+                f"{size} bytes requested, {len(data)} bytes returned."
+            )
         return data
 
     def _read_line(self) -> bytes:
@@ -311,28 +313,34 @@ class UBXReader:
 
         :return: bytes
         :rtype: bytes
-        :raises: EOFError if stream ends prematurely
+        :raises: UBXStreamError if stream ends prematurely
         """
 
         data = self._stream.readline()  # NMEA protocol is CRLF-terminated
-        if data[-1:] != b"\x0a":
-            raise EOFError()
+        if len(data) == 0:
+            raise EOFError()  # pragma: no cover
+        if data[-1:] != b"\x0a":  # truncated stream
+            raise UBXStreamError(
+                "Serial stream terminated unexpectedly. "
+                f"Line requested, {len(data)} bytes returned."
+            )
         return data
 
-    def _do_error(self, err: str):
+    def _do_error(self, err: Exception):
         """
         Handle error.
 
-        :param str err: error message
-        :raises: UBXParseError if quitonerror = 2
+        :param Exception err: error
+        :raises: Exception if quitonerror = ERR_RAISE (2)
         """
 
         if self._quitonerror == ERR_RAISE:
-            raise UBXParseError(err)
+            raise err from err
         if self._quitonerror == ERR_LOG:
             # pass to error handler if there is one
+            # else just log
             if self._errorhandler is None:
-                print(err)
+                self._logger.error(err)
             else:
                 self._errorhandler(err)
 
@@ -352,30 +360,26 @@ class UBXReader:
         message: bytes,
         msgmode: int = GET,
         validate: int = VALCKSUM,
-        quitonerror: int = ERR_LOG,
         parsebitfield: bool = True,
-        scaling: bool = True,
     ) -> object:
         """
         Parse UBX byte stream to UBXMessage object.
 
-        Includes option to validate incoming payload length and checksum
-        (the UBXMessage constructor can calculate and assign its own values anyway).
-
         :param bytes message: binary message to parse
-        :param int quitonerror: 0 = ignore errors,  1 = log continue, 2 = (re)raise (1)
-        :param int validate: validate cksum (VALCKSUM (1)=True (default), VALNONE (0)=False)
-        :param int msgmode: message mode (0=GET (default), 1=SET, 2=POLL)
+        :param int msgmode: GET (0), SET (1), POLL (2) (0)
+        :param int validate: VALCKSUM (1) = Validate checksum,
+            VALNONE (0) = ignore invalid checksum (1)
         :param bool parsebitfield: 1 = parse bitfields, 0 = leave as bytes (1)
-        :param bool scaling: 1 = apply scale factors, 0 = do not apply (1)
         :return: UBXMessage object
         :rtype: UBXMessage
-        :raises: UBXParseError (if data stream contains invalid data or unknown message type)
+        :raises: Exception (if data stream contains invalid data or unknown message type)
         """
         # pylint: disable=too-many-arguments
 
-        if msgmode not in (0, 1, 2):
-            raise UBXParseError(f"Invalid message mode {msgmode} - must be 0, 1 or 2")
+        if msgmode not in (GET, SET, POLL, SETPOLL):
+            raise UBXParseError(
+                f"Invalid message mode {msgmode} - must be 0, 1, 2 or 3"
+            )
 
         lenm = len(message)
         hdr = message[0:2]
@@ -409,25 +413,15 @@ class UBXReader:
                 raise UBXParseError(
                     (f"Message checksum {ckm}" f" invalid - should be {ckv}")
                 )
-        try:
-            if payload is None:
-                return UBXMessage(clsid, msgid, msgmode)
-            return UBXMessage(
-                clsid,
-                msgid,
-                msgmode,
-                payload=payload,
-                parsebitfield=parsebitfield,
-                scaling=scaling,
-            )
-        except KeyError as err:
-            modestr = ["GET", "SET", "POLL"][msgmode]
-            errmsg = (
-                f"Unknown message type clsid {clsid}, msgid {msgid}, mode {modestr}\n"
-                + "Check 'msgmode' keyword argument is appropriate for data stream"
-            )
-            if quitonerror == ERR_RAISE:
-                raise UBXParseError(errmsg) from err
-            if quitonerror == ERR_LOG:
-                print(errmsg)
-            return None
+        # if input message (SET or POLL), determine mode automatically
+        if msgmode == SETPOLL:
+            msgmode = getinputmode(message)  # returns SET or POLL
+        if payload is None:
+            return UBXMessage(clsid, msgid, msgmode)
+        return UBXMessage(
+            clsid,
+            msgid,
+            msgmode,
+            payload=payload,
+            parsebitfield=parsebitfield,
+        )
